@@ -7,7 +7,7 @@ from chia.util.bech32m import encode_puzzle_hash
 from chia.util.config import load_config
 from chia.util.default_root import DEFAULT_ROOT_PATH
 from chia.util.ints import uint32
-from chia.util.keychain import Keychain, bytes_to_mnemonic, generate_mnemonic
+from chia.util.keychain import Keychain, bytes_to_mnemonic, generate_mnemonic, unlocks_keyring
 from chia.wallet.derive_keys import master_sk_to_farmer_sk, master_sk_to_pool_sk, master_sk_to_wallet_sk
 
 keychain: Keychain = Keychain()
@@ -25,6 +25,7 @@ def generate_and_print():
     return mnemonic
 
 
+@unlocks_keyring(use_passphrase_cache=True)
 def generate_and_add():
     """
     Generates a seed for a private key, prints the mnemonic to the terminal, and adds the key to the keyring.
@@ -35,11 +36,13 @@ def generate_and_add():
     add_private_key_seed(mnemonic)
 
 
+@unlocks_keyring(use_passphrase_cache=True)
 def query_and_add_private_key_seed():
     mnemonic = input("Enter the mnemonic you want to use: ")
     add_private_key_seed(mnemonic)
 
 
+@unlocks_keyring(use_passphrase_cache=True)
 def add_private_key_seed(mnemonic: str):
     """
     Add a private key seed to the keyring, with the given mnemonic.
@@ -56,6 +59,7 @@ def add_private_key_seed(mnemonic: str):
         return None
 
 
+@unlocks_keyring(use_passphrase_cache=True)
 def show_all_keys(show_mnemonic: bool):
     """
     Prints all keys and mnemonics (if available).
@@ -97,6 +101,7 @@ def show_all_keys(show_mnemonic: bool):
             print(mnemonic)
 
 
+@unlocks_keyring(use_passphrase_cache=True)
 def delete(fingerprint: int):
     """
     Delete a key by its public key fingerprint (which is an integer).
@@ -105,7 +110,8 @@ def delete(fingerprint: int):
     keychain.delete_key_by_fingerprint(fingerprint)
 
 
-def sign(message: str, fingerprint: int, hd_path: str):
+@unlocks_keyring(use_passphrase_cache=True)
+def sign(message: str, fingerprint: int, hd_path: str, as_bytes: bool):
     k = Keychain()
     private_keys = k.get_all_private_keys()
 
@@ -114,8 +120,9 @@ def sign(message: str, fingerprint: int, hd_path: str):
         if sk.get_g1().get_fingerprint() == fingerprint:
             for c in path:
                 sk = AugSchemeMPL.derive_child_sk(sk, c)
+            data = bytes.fromhex(message) if as_bytes else bytes(message, "utf-8")
             print("Public key:", sk.get_g1())
-            print("Signature:", AugSchemeMPL.sign(sk, bytes(message, "utf-8")))
+            print("Signature:", AugSchemeMPL.sign(sk, data))
             return None
     print(f"Fingerprint {fingerprint} not found in keychain")
 
