@@ -10,7 +10,7 @@ def create_puzzlehash_for_pk(pub_key: G1Element) -> bytes32:
     return puzzle_for_pk(pub_key).get_tree_hash()
 
 
-def pool_parent_id(block_height: uint32, genesis_challenge: bytes32) -> uint32:
+def pool_parent_id(block_height: uint32, genesis_challenge: bytes32) -> bytes32:
     return bytes32(genesis_challenge[:16] + block_height.to_bytes(16, "big"))
 
 
